import {Component, h, Prop, Host, Element} from '@stencil/core';
import {css} from 'emotion';

/*
Example of using hy-box:
<hy-box mb="1.75, 1.75, 2, 2.5" />
Where mb = margin-bottom,
mobile up to 30rem: 1.75rem (mb[0])
tablet, 30rem - 60rem: 1.75rem (mb[1])
wide, 60rem to 75rem or 60rem to 100.0625rem (depending on hasSidebar) (mb[2])
xlarge, > 100.0625: 2.5rem (mb[3])
* */
const toRemValue = (val?: number) => {
  return typeof val === 'number' ? `${val}rem` : undefined;
};

interface P {
  hasSidebar?: boolean;
  p?: number[];
  pt?: number[];
  pl?: number[];
  pb?: number[];
  pr?: number[];
  m?: number[];
  mt?: number[];
  ml?: number[];
  mb?: number[];
  mr?: number[];
  h?: number;
  bg?: string;
  width: string[];
  justify: string;
  align: string;
}

const breakpoints = ['30rem', '60rem', '75.0625rem', '100.0625rem'];

/*
Default - mobile
mq[0] - tablet
mq[1] - wide, >960px
mq[2] - extrawide, > 1200px
mq[3] - xlarge, >1600px
* */
const mq = breakpoints.map((b) => `@media (min-width: ${b})`);
const createCssClass = ({hasSidebar, pt, pb, pl, pr, mt, mb, ml, mr, h, bg, width, justify, align}: P) => {
  const last = width[width.length - 1];

  return css`
    padding-top: ${toRemValue(pt[0])};
    padding-right: ${toRemValue(pr[0])};
    padding-bottom: ${toRemValue(pb[0])};
    padding-left: ${toRemValue(pl[0])};
    margin-top: ${toRemValue(mt[0])};
    margin-right: ${toRemValue(mr[0])};
    margin-bottom: ${toRemValue(mb[0])};
    margin-left: ${toRemValue(ml[0])};
    height: ${toRemValue(h)};
    background-color: ${!!bg ? `var(--${bg})` : 'transparent'};
    width: ${width[0]};
    display: flex;
    justify-content: ${justify || 'flex-start'};
    align-items: ${align || 'flex-start'};

    ${mq[0]} {
      width: ${width[1]};

      padding-top: ${toRemValue(pt[1])};
      padding-right: ${toRemValue(pr[1])};
      padding-bottom: ${toRemValue(pb[1])};
      padding-left: ${toRemValue(pl[1])};

      margin-top: ${toRemValue(mt[1])};
      margin-right: ${toRemValue(mr[1])};
      margin-bottom: ${toRemValue(mb[1])};
      margin-left: ${toRemValue(ml[1])};
    }
    ${mq[1]} {
      width: ${width[2] || last};

      padding-top: ${toRemValue(pt[2])};
      padding-right: ${toRemValue(pr[2])};
      padding-bottom: ${toRemValue(pb[2])};
      padding-left: ${toRemValue(pl[2])};

      margin-top: ${toRemValue(mt[2])};
      margin-right: ${toRemValue(mr[2])};
      margin-bottom: ${toRemValue(mb[2])};
      margin-left: ${toRemValue(ml[2])};
    }
    ${mq[2]} {
      width: ${width[3] || last};

      padding-top: ${hasSidebar ? `${toRemValue(pt[2])}` : `${toRemValue(pt[3])}`};
      padding-right: ${hasSidebar ? `${toRemValue(pr[2])}` : `${toRemValue(pr[3])}`};
      padding-bottom: ${hasSidebar ? `${toRemValue(pb[2])}` : `${toRemValue(pb[3])}`};
      padding-left: ${hasSidebar ? `${toRemValue(pl[2])}` : `${toRemValue(pl[3])}`};

      margin-top: ${hasSidebar ? `${toRemValue(mt[2])}` : `${toRemValue(mt[3])}`};
      margin-right: ${hasSidebar ? `${toRemValue(mr[2])}` : `${toRemValue(mr[3])}`};
      margin-bottom: ${hasSidebar ? `${toRemValue(mb[2])}` : `${toRemValue(mb[3])}`};
      margin-left: ${hasSidebar ? `${toRemValue(ml[2])}` : `${toRemValue(ml[3])}`};
    }
    ${mq[3]} {
      width: ${width[4] || last};

      padding-top: ${toRemValue(pt[3])};
      padding-right: ${toRemValue(pr[3])};
      padding-bottom: ${toRemValue(pb[3])};
      padding-left: ${toRemValue(pl[3])};

      margin-top: ${toRemValue(mt[3])};
      margin-right: ${toRemValue(mr[3])};
      margin-bottom: ${toRemValue(mb[3])};
      margin-left: ${toRemValue(ml[3])};
    }
  `;
};

const toNumArray = (value?: string) =>
  value
    ?.split(',')
    .map((v) => `${v.trim()}`)
    .map((v) => parseFloat(v));

@Component({
  tag: 'hy-box',
  styleUrl: 'hy-box.scss',
  shadow: false,
})
export class HyBox {
  @Element() el: HTMLElement;
  @Prop() headerstyle: string = 'common';
  /**
   * All sides padding value in rems. Responsive when given Comma separated values.
   */
  @Prop() p?: string;

  /**
   * Left padding in rems. Responsive when given Comma separated values.
   */
  @Prop() pl?: string;
  /**
   * Right padding in rems. Responsive when given Comma separated values.
   */
  @Prop() pr?: string;
  /**
   * Bottom padding in rems. Responsive when given Comma separated values.
   */
  @Prop() pb?: string;
  /**
   * Top padding in rems. Responsive when given Comma separated values.
   */
  @Prop() pt?: string;
  /**
   * All sides margin value in rems. Responsive when given Comma separated values.
   */
  @Prop() m?: string;

  /**
   * Left margin in rems. Responsive when given Comma separated values.
   */
  @Prop() ml?: string;
  /**
   * Right margin in rems. Responsive when given Comma separated values.
   */
  @Prop() mr?: string;
  /**
   * Bottom margin in rems. Responsive when given Comma separated values.
   */
  @Prop() mb?: string;
  /**
   * Top margin in rems. Responsive when given Comma separated values.
   */
  @Prop() mt?: string;
  /**
   * Value for horisontal alignment (justify-content)
   */
  @Prop() justify: string = 'flex-start';
  /**
   * value for vertical alignment (align-items)
   */
  @Prop() align: string = 'baseline';

  /**
   * fixed height for the container, useful for extra spacing containers
   */
  @Prop() h?: number;
  /**
   * background color. Must use one of css variable names from color tokens
   */
  @Prop() bg: string = 'transparent';
  /**
   * Responsive width. Must be a comma separated string of percentage values for breakpoints. First item is the default value.
   */
  @Prop() width: string = '100';

  componentWillLoad() {
    let hyMainDiv = this.el.closest('.hy-main');
    if (hyMainDiv) {
      if (!hyMainDiv.classList.contains('with-sidebar')) {
        this.headerstyle = 'large';
      }
    }
  }

  render() {
    const pt = toNumArray(this.pt) ?? toNumArray(this.p) ?? [0];
    const pb = toNumArray(this.pb) ?? toNumArray(this.p) ?? [0];
    const pr = toNumArray(this.pr) ?? toNumArray(this.p) ?? [0];
    const pl = toNumArray(this.pl) ?? toNumArray(this.p) ?? [0];
    const mt = toNumArray(this.mt) ?? toNumArray(this.m) ?? [0];
    const mb = toNumArray(this.mb) ?? toNumArray(this.m) ?? [0];
    const mr = toNumArray(this.mr) ?? toNumArray(this.m) ?? [0];
    const ml = toNumArray(this.ml) ?? toNumArray(this.m) ?? [0];
    const hasSidebar = this.headerstyle == 'common';

    return (
      <Host
        class={createCssClass({
          hasSidebar,
          pt,
          pl,
          pr,
          pb,
          mt,
          ml,
          mr,
          mb,
          h: this.h,
          bg: this.bg,
          width: this.width.split(',').map((v) => `${v.trim()}%`),
          justify: this.justify,
          align: this.align,
        })}
      >
        <slot></slot>
      </Host>
    );
  }
}
